package com.zhili.dashboard.controller;


import com.zhili.dashboard.base.*;
import com.zhili.dashboard.entity.CmdDetail;
import com.zhili.dashboard.req.MapParam;
import com.zhili.dashboard.req.ProvinceParam;
import com.zhili.dashboard.req.RadarParam;
import com.zhili.dashboard.service.IAlarmMsgService;
import com.zhili.dashboard.vo.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;

/**
 * 故障信息表
 *
 * @author zyg
 * @since 2022-07-05
 */
@Api(tags = "故障信息表")
@RestController
@RequestMapping("/dashboard/alarmMsg")
public class AlarmMsgController {

    @Resource
    IAlarmMsgService alarmMsgService;

    //@Cacheable(cacheNames = "gatewayHandler", key = "'getGatewayHandler'")
    @ApiOperation("报警统计")
    @GetMapping("/level/{type}")
    public AjaxResult getAlarmLevelPie(@PathVariable("type") Integer type) {
        List<BaseCycleChartVo> result = alarmMsgService.getAlarmLevelPie(type);
        return AjaxResult.success(result);
    }

    @ApiOperation("故障预警历史信息-24小时")
    @GetMapping("/history/day")
    public AjaxResult getAlarmHistoryLineForDay() {
        BaseLineDecimalVo result = alarmMsgService.getAlarmHistoryLineForDay();
        return AjaxResult.success(result);
    }

    @ApiOperation("故障预警历史信息-30天")
    @GetMapping("/history/month")
    public AjaxResult getAlarmHistoryLineForMonth() {
        BaseLineDecimalVo result = alarmMsgService.getAlarmHistoryLineForMonth();
        return AjaxResult.success(result);
    }

    @ApiOperation("高危用户列表")
    @GetMapping("/highRisk")
    public AjaxResult getHighRisk() {
        List<HighRiskVo> result = alarmMsgService.getHighRisk();
        return AjaxResult.success(result);
    }

    @ApiOperation("风险等级电池统计信息")
    @GetMapping("/riskList")
    public AjaxResult getRiskList() {
        List<RiskListVo> result = alarmMsgService.getRiskList();
        return AjaxResult.success(result);
    }

    @ApiOperation("故障电池与正常电池统计信息")
    @GetMapping("/faultList")
    public AjaxResult getFaultList() {
        List<BaseCycleChartVo> result = alarmMsgService.getFaultList();
        return AjaxResult.success(result);
    }

    @ApiOperation("在线电池统计信息")
    @GetMapping("/onLine")
    public AjaxResult getOnLine() {
        OnLineVo result = alarmMsgService.getOnLine();
        return AjaxResult.success(result);
    }

    @ApiOperation("电池状态统计信息")
    @GetMapping("/deviceStatusList")
    public AjaxResult getDeviceStatusList() {
        BaseLineDecimalVo result = alarmMsgService.getDeviceStatusList();
        return AjaxResult.success(result);
    }

    @ApiOperation("区域电池信息排名")
    @PostMapping("/districtOrder")
    public AjaxResult getDistrictOrder(@RequestBody ProvinceParam param) {
        List<SceneGroupVo> result = alarmMsgService.getDistrictOrder(param);
        return AjaxResult.success(result);
    }

    @ApiOperation("应用场景电池信息")
    @PostMapping("/sceneGroup")
    public AjaxResult getSceneGroup(@RequestBody ProvinceParam param) {
        List<SceneGroupVo> result = alarmMsgService.getSceneGroup(param);
        return AjaxResult.success(result);
    }

    @ApiOperation("累计安全拦截次数&安全运行时长")
    @GetMapping("/safeguard")
    public AjaxResult getSafeguard() {
        SafeguardVo result = alarmMsgService.getSafeguard();
        return AjaxResult.success(result);
    }

    @ApiOperation("累计信息统计")
    @GetMapping("/cumulativeInfo")
    public AjaxResult getCumulativeInfo() {
        CumulativeInfoVo result = alarmMsgService.getCumulativeInfo();
        return AjaxResult.success(result);
    }

    @ApiOperation("累计循环次数")
    @GetMapping("/cumulativeCycle")
    public AjaxResult getCumulativeCycle() {
        Long result = alarmMsgService.getCumulativeCycle();
        return AjaxResult.success(result);
    }

    @ApiOperation("电池详情")
    @GetMapping("/detail/{id}")
    public AjaxResult getDetail(@PathVariable("id") Long id) {
        AlarmDetailVo result = alarmMsgService.getDetail(id);
        return AjaxResult.success(result);
    }

    @ApiOperation("雷达图")
    @PostMapping("/radar")
    public AjaxResult getRadar(@RequestBody RadarParam param) {
        KindValueDoubleListVo result = alarmMsgService.getRadar(param);
        return AjaxResult.success(result);
    }

    @ApiOperation("地图信息")
    @PostMapping("/batteryMap")
    public AjaxResult getBatteryMap(@RequestBody MapParam param) {
        List<BatteryMapVo> result = alarmMsgService.getBatteryMap(param);
        return AjaxResult.success(result);
    }

    @ApiOperation("安全事故率")
    @PostMapping("/faultRate")
    public AjaxResult getFaultRate(@RequestBody MapParam param) {
        BigDecimal result = alarmMsgService.getFaultRate(param);
        return AjaxResult.success(result);
    }

    @ApiOperation("安全事故列表")
    @PostMapping("/faultRateList")
    public AjaxResult getFaultRateList(@RequestBody MapParam param) {
        BaseLineDecimalVo result = alarmMsgService.getFaultRateList(param);
        return AjaxResult.success(result);
    }

    @ApiOperation("安全明细")
    @GetMapping("/cmdDetailList")
    public AjaxResult getCmdDetailList() {
        List<CmdDetail> result = alarmMsgService.getCmdDetailList();
        return AjaxResult.success(result);
    }

    @ApiOperation("安全概览")
    @GetMapping("/cmdOverview")
    public AjaxResult getCmdOverview() {
        List<BaseCycleChartVo> result = alarmMsgService.getCmdOverview();
        return AjaxResult.success(result);
    }
}
