package com.zhili.dashboard.handler;

import com.baomidou.mybatisplus.core.handlers.MetaObjectHandler;
import com.zhili.dashboard.base.UserInformationDTO;
import com.zhili.dashboard.utils.GetUserInformationUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.ibatis.reflection.MetaObject;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;

/**
 * @ClassName : MyMetaObjectHandler
 * @Description : 自动插入数据
 * @Component * 一定不要忘记把处理器加入到IOC容器中。
 * @Author : zyg
 * @Date: 2021-05-14 10:41
 */
@Slf4j
@Component
public class CustomMetaObjectHandler implements MetaObjectHandler {

    /**
     * 插入时候的填充策略
     *
     * @param metaObject 字段
     */
    @Override
    public void insertFill(MetaObject metaObject) {
        UserInformationDTO userBasic = GetUserInformationUtils.getUserBasic();
        LocalDateTime now = LocalDateTime.now();
        this.setFieldValByName("createTime", now, metaObject);
        this.setFieldValByName("updateTime", now, metaObject);
        if (null != userBasic) {
            this.setFieldValByName("create_by", userBasic.getName(), metaObject);
            this.setFieldValByName("update_by", userBasic.getName(), metaObject);
        } else {
            this.setFieldValByName("create_by", "系统自动执行", metaObject);
            this.setFieldValByName("update_by", "系统自动执行", metaObject);
        }
    }

    /**
     * 更新时间的填充策略
     */
    @Override
    public void updateFill(MetaObject metaObject) {
        UserInformationDTO userBasic = GetUserInformationUtils.getUserBasic();
        if (null != userBasic) {
            this.setFieldValByName("create_by", userBasic.getName(), metaObject);
        } else {
            this.setFieldValByName("update_by", "系统自动执行", metaObject);
        }
        this.setFieldValByName("updateTime", LocalDateTime.now(), metaObject);
    }

}

