package com.zhili.dashboard.mapper;

import com.baomidou.mybatisplus.annotation.InterceptorIgnore;
import com.zhili.dashboard.entity.AlarmMsg;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.zhili.dashboard.entity.Device;
import com.zhili.dashboard.entity.DeviceDuration;
import org.apache.ibatis.annotations.MapKey;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * 云控信息表 Mapper 接口
 * </p>
 *
 * @author zyg
 * @since 2022-07-05
 */
public interface AlarmMsgMapper extends BaseMapper<AlarmMsg> {

    /**
     * 按照故障等级分组查询
     * @param dateStr 参数
     * @return java.util.List<com.zhili.dashboard.entity.AlarmMsg>
     */
    public List<AlarmMsg> getAlarmLevelPie(@Param("dateStr") String dateStr);

    /**
     * 最近24小时故障数量统计
     * @param dateStr 参数
     * @return java.util.List<Map<String, Object>>
     */
    @MapKey("id")
    public List<Map<String, Object>> getAlarmHistoryDay(@Param("dateStr") String dateStr);

    /**
     * 最近24小时故障完结量统计
     * @param dateStr 参数
     * @return java.util.List<Map<String, Object>>
     */
    @MapKey("id")
    public List<Map<String, Object>> getAlarmHistoryDayOverRate(@Param("dateStr") String dateStr);

    /**
     * 最近30天故障数量统计
     * @param dateStr 参数
     * @return List<Map<String, Object>>
     */
    @MapKey("id")
    public List<Map<String, Object>> getAlarmHistoryMonth(@Param("dateStr") String dateStr);

    /**
     * 最近30天故障完结率
     * @param dateStr 参数
     * @return List<Map<String, Object>>
     */
    @MapKey("id")
    public List<Map<String, Object>> getAlarmHistoryOveMonthRate(@Param("dateStr") String dateStr);





    /**
     * 获取高危用户
     * @return java.util.List<java.util.Map<java.lang.String,java.lang.Object>>
     */
    @MapKey("id")
    public List<Map<String, Object>> getDeviceStatusGroup();

    /**
     * 在线电池统计
     * @return java.util.List<java.util.Map<java.lang.String,java.lang.Object>>
     */
    @MapKey("id")
    List<Map<String, Object>> getOnLine();

    /**
     * 电池状态统计信息
     * @return java.util.List<java.util.Map<java.lang.String,java.lang.Object>>
     */
    @MapKey("id")
    List<Map<String, Object>> getDeviceStatusList(@Param("dateStr") String dateStr);

    /**
     * 报警数量
     * @param province 参数
     * @return java.util.List<java.util.Map<java.lang.String,java.lang.Object>>
     */
    @MapKey("id")
    List<Map<String, Object>> getDistrictFaultOrder(@Param("province") String province);

    /**
     * 电池数量
     * @param province 参数
     * @return java.util.List<java.util.Map<java.lang.String,java.lang.Object>>
     */
    @MapKey("id")
    List<Map<String, Object>> getDistrictDeviceOrder(@Param("province") String province);

    /**
     * 电池数量
     * @param province 参数
     * @return java.util.List<java.util.Map<java.lang.String,java.lang.Object>>
     */
    @MapKey("id")
    List<Map<String, Object>> getDistrictDeviceOrderSix(@Param("province") String province);

    /**
     * 应用场景电池信息
     * @param province 参数
     * @return java.util.List<java.util.Map<java.lang.String,java.lang.Object>>
     */
    @MapKey("id")
    List<Map<String, Object>> getSceneFaultGroup(@Param("province") String province);

    /**
     * 应用场景电池信息
     * @param province 参数
     * @return java.util.List<java.util.Map<java.lang.String,java.lang.Object>>
     */
    @MapKey("id")
    List<Map<String, Object>> getSceneDeviceGroup(@Param("province") String province);

    /**
     * 安全运行时间
     * @return java.lang.Integer
     */
    Long getInterval();

    /**
     * 累计信息统计
     * @return java.util.Map<java.lang.String,java.lang.Object>
     */
    @MapKey("id")
    List<Map<String, Object>> getCumulativeInfo();

    /**
     * 电池详情
     * @param id 参数
     * @return List<Map<String, Object>>
     */
    @MapKey("id")
    List<Map<String, Object>> getDetail(@Param("id") Long id);

    /**
     * 雷达图
     * @param sn 参数
     * @return List<Map<String, Object>>
     */
    @MapKey("id")
    List<Map<String, Object>> getRadarItems(@Param("sn") String sn);

    /**
     * 电池地图
     * @param province 参数
     * @return List<Map<String, Object>>
     */
    @MapKey("id")
    List<Map<String, Object>> getBatteryMap(@Param("province") String province);
    @InterceptorIgnore(blockAttack = "true")
    void updateAlarmMsg();
}
