package com.zhili.dashboard.scheduler;

import com.zhili.dashboard.service.*;
import lombok.extern.log4j.Log4j2;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * @author user
 */
@Component
@Log4j2
public class JobScheduler {
    @Resource
    IDeviceService deviceService;
    @Resource
    IAlarmMsgService alarmMsgService;
    @Resource
    IAlarmMsgWarnService alarmMsgWarnService;
    @Resource
    IDeviceDurationService deviceDurationService;
    @Resource
    IStatisticsService statisticsService;


    /**
     * 同步电池信息
     * 10min一次
     */
    @Scheduled(cron = "0 0 0 * * ?")
    public void updateAlarmMsg() {
        log.info("更新一次数据");
        alarmMsgService.updateAlarmMsg();
    }
    /**
     * 同步电池信息
     * 10min一次
     */
    @Async
    @Scheduled(cron = "0 */10 * * * ?")
    public void syncBattery() {
        deviceService.syncBattery();
    }

    /**
     * 同步报警信息
     * 10min一次
     */
    @Async
    //@Scheduled(cron = "0 */2 * * * ?")
    public void syncAlarmMsg() {
        alarmMsgService.syncAlarmMsg();
    }

    /**
     * 同步预警信息
     * 2min一次
     */
    @Async
    //@Scheduled(cron = "0 */2 * * * ?")
    public void syncAlarmMsgWarnIng() {
        alarmMsgWarnService.syncAlarmMsg();
    }

    /**
     * 更新故障状态
     * 1小时一次
     */
    @Async
    @Scheduled(cron = "0 0 */1 * * ?")
    public void syncAlarmMsgStatus() {
        alarmMsgService.syncAlarmMsgStatus();
    }

    /**
     * 同步析锂时长信息
     * 1小时一次
     */
    @Async
    @Scheduled(cron = "0 0 */1 * * ?")
    public void syncDuration() {
        deviceDurationService.syncDuration();
    }

    @Async
    @Scheduled(cron = "0 */3 * * * ?")
    public void updateDevice(){
        deviceService.updateDevice();
    }

    @Async
    @Scheduled(cron = "0 */3 * * * ?")
    public void updateStatistics(){
        statisticsService.updateStatistics();
    }
}
