package com.zhili.dashboard.service;

import com.zhili.dashboard.base.*;
import com.zhili.dashboard.entity.AlarmMsg;
import com.baomidou.mybatisplus.extension.service.IService;
import com.zhili.dashboard.entity.CmdDetail;
import com.zhili.dashboard.req.MapParam;
import com.zhili.dashboard.req.ProvinceParam;
import com.zhili.dashboard.req.RadarParam;
import com.zhili.dashboard.vo.*;

import java.math.BigDecimal;
import java.util.List;

/**
 * <p>
 * 云控信息表 服务类
 * </p>
 *
 * @author zyg
 * @since 2022-07-05
 */
public interface IAlarmMsgService extends IService<AlarmMsg> {
    /**
     * 同步电池信息
     */
    void syncAlarmMsg();

    /**
     * 更新故障信息
     */
    void syncAlarmMsgStatus();

    /**
     * 故障级别饼图
     *
     * @param type 参数
     * @return java.util.List<com.zhili.dashboard.base.BaseCycleChartVo>
     */
    List<BaseCycleChartVo> getAlarmLevelPie(Integer type);

    /**
     * 故障预警历史信息-24小时
     *
     * @return com.zhili.dashboard.base.BaseLineDecimalVo
     */
    BaseLineDecimalVo getAlarmHistoryLineForDay();

    /**
     * 故障预警历史信息-30天
     *
     * @return com.zhili.dashboard.base.BaseLineDecimalVo
     */
    BaseLineDecimalVo getAlarmHistoryLineForMonth();

    /**
     * 高危用户列表
     *
     * @return List<HighRiskVo>
     */
    List<HighRiskVo> getHighRisk();

    /**
     * 风险等级电池统计信息
     *
     * @return java.util.List<com.zhili.dashboard.vo.RiskListVo>
     */
    List<RiskListVo> getRiskList();

    /**
     * 故障电池与正常电池统计信息
     *
     * @return java.util.List<BaseCycleChartVo>
     */
    List<BaseCycleChartVo> getFaultList();

    /**
     * 在线电池统计信息
     *
     * @return java.util.List<com.zhili.dashboard.base.BaseCycleChartVo>
     */
    OnLineVo getOnLine();

    /**
     * 电池状态统计信息
     *
     * @return BaseLineDecimalVo
     */
    BaseLineDecimalVo getDeviceStatusList();

    /**
     * 区域电池信息排名(仅北京市)
     * @param param 参数
     * @return java.util.List<com.zhili.dashboard.vo.SceneGroupVo>
     */
    List<SceneGroupVo> getDistrictOrder(ProvinceParam param);

    /**
     * 应用场景电池信息
     *
     * @param param 参数
     * @return java.util.List<com.zhili.dashboard.vo.SceneGroupVo>
     */
    List<SceneGroupVo> getSceneGroup(ProvinceParam param);

    /**
     * 累计安全拦截次数&安全运行时长
     *
     * @return SafeguardVo
     */
    SafeguardVo getSafeguard();

    /**
     * 累计信息统计
     *
     * @return com.zhili.dashboard.vo.CumulativeInfoVo
     */
    CumulativeInfoVo getCumulativeInfo();

    /**
     * 累计循环次数
     * @return java.lang.Long
     */
    Long getCumulativeCycle();

    /**
     * 电池详情
     * @param id 参数
     * @return AlarmDetailVo
     */
    AlarmDetailVo getDetail(Long id);

    /**
     * 雷达图
     * @param param 参数
     * @return KindValueListVo
     */
    KindValueDoubleListVo getRadar(RadarParam param);

    /**
     * 地图信息
     * @param param 参数
     * @return java.util.List<BatteryMapVo>
     */
    List<BatteryMapVo> getBatteryMap(MapParam param);

    /**
     * 安全事故率
     * @param param 参数
     * @return java.math.BigDecimal
     */
    BigDecimal getFaultRate(MapParam param);

    /**
     * 安全事故率
     * @param param 参数
     * @return BaseLineDecimalVo
     */
    BaseLineDecimalVo getFaultRateList(MapParam param);

    /**
     * 安全明细
     * @return java.util.List<CmdDetail>
     */
    List<CmdDetail> getCmdDetailList();

    /**
     * 安全概览
     * @return java.util.List<BaseCycleChartVo>
     */
    List<BaseCycleChartVo> getCmdOverview();

    void updateAlarmMsg();
}
