package com.zhili.dashboard.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zhili.dashboard.base.BaseCycleChartVo;
import com.zhili.dashboard.base.BaseLineDecimalVo;
import com.zhili.dashboard.base.KindValueDoubleListVo;
import com.zhili.dashboard.entity.*;
import com.zhili.dashboard.enums.*;
import com.zhili.dashboard.mapper.*;
import com.zhili.dashboard.req.RadarParam;
import com.zhili.dashboard.req.TableInfoParam;
import com.zhili.dashboard.service.IAlarmMsgService;
import com.zhili.dashboard.service.IAlarmMsgWarnService;
import com.zhili.dashboard.utils.DateUtil;
import com.zhili.dashboard.utils.LineTimeChartVo;
import com.zhili.dashboard.utils.OkHttpCli;
import com.zhili.dashboard.utils.StringUtils;
import com.zhili.dashboard.vo.*;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.AsyncResult;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.*;
import java.util.concurrent.Future;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.stream.Collectors;

/**
 * <p>
 * 云控信息表 服务实现类
 * </p>
 *
 * @author zyg
 * @since 2022-07-05
 */
@Service
public class AlarmMsgWarnServiceImpl extends ServiceImpl<AlarmMsgWarnMapper, AlarmMsgWarn> implements IAlarmMsgWarnService {
    @Resource
    OkHttpCli okHttpCli;
    @Resource
    AlarmMsgMapper alarmMsgMapper;
    @Resource
    DeviceMapper deviceMapper;
    @Resource
    DistributionMapMapper distributionMapMapper;
    @Resource
    DeviceDurationMapper deviceDurationMapper;
    @Resource
    ModelMapper modelMapper;
    @Resource
    OnlineMapper onlineMapper;
    @Resource
    AlarmMsgWarnMapper alarmMsgWarnMapper;
    /**
     * 数据获取地址
     */
    private static final String BATTERY_URL = "/admin/v1/hzBattery";
    /**
     * 故障信息地址
     */
    private static final String SIGNATURE = "AD5C4EC001352FFAC3A03CBA787A7417";

    @Value("${openapi.token-secret}")
    private String tokenSecret;
    @Value("${openapi.url}")
    private String openApiUrl;


    /**
     * 同步故障信息
     * 获取上周合众数据，注意间隔时间为7天
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void syncAlarmMsg() {

        int interval = 5;
        long maxId = 0L;
        long maxId1 = 0L;
        QueryWrapper<AlarmMsgWarn> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_delete", 0);
        queryWrapper.eq("type", 1);
        queryWrapper.select("max(sid) as id");
        AlarmMsgWarn alarmMsg = alarmMsgWarnMapper.selectOne(queryWrapper);
        if (alarmMsg != null && alarmMsg.getId() != null) {
            maxId = alarmMsg.getId();
        }

        QueryWrapper<AlarmMsgWarn> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("is_delete", 0);
        queryWrapper1.eq("type", 2);
        queryWrapper1.select("max(sid) as id");
        AlarmMsgWarn alarmMsg1 = alarmMsgWarnMapper.selectOne(queryWrapper1);
        if (alarmMsg1 != null && alarmMsg1.getId() != null) {
            maxId1 = alarmMsg1.getId();
        }


        //获取电池信息
        List<String> response = new ArrayList<>();
        try {
            List<Future<String>> futures = new ArrayList<>();
            TableInfoParam param = new TableInfoParam();
            param.setSignature(SIGNATURE);
            param.setInterfaceName("设备基本信息");
            param.setCondition("date_info <= DATE_ADD(now(), INTERVAL -" + interval + " DAY) and id>" + maxId + " order by id ");
            param.setTableName("algo_all_fault_info_ing");
            param.setFields("id,sn,fault_code,start_time,end_time,fault_advice,update_by");

            TableInfoParam paramDone = new TableInfoParam();
            paramDone.setSignature(SIGNATURE);
            paramDone.setInterfaceName("设备基本信息");
            paramDone.setCondition("date_info <= DATE_ADD(now(), INTERVAL -" + interval + " DAY) and id>" + maxId1 + " order by id ");
            paramDone.setTableName("algo_all_fault_info_done");
            paramDone.setFields("id,sn,fault_code,start_time,end_time,fault_advice,update_by");

            futures.add(getBatteryMsg(openApiUrl + BATTERY_URL, JSON.toJSONString(param)));
            futures.add(getBatteryMsg(openApiUrl + BATTERY_URL, JSON.toJSONString(paramDone)));
            //死循环，每隔2000ms执行一次，判断一下这三个异步调用的方法是否全都执行完了。
            while (true) {
                //使用Future的isDone()方法返回该方法是否执行完成
                if (futures.get(0).isDone() && futures.get(1).isDone()) {
                    //如果异步方法全部执行完，跳出循环
                    break;
                }
                //每隔200毫秒判断一次
                Thread.sleep(200);
            }
            for (Future<String> future : futures) {
                String string = future.get();
                response.add(string);
            }
        } catch (Exception e) {
            return;
        }
        // 故障信息解析
        saveAlarmMsgWarn(response, interval);
    }

    @Transactional(rollbackFor = Exception.class)
    public void saveAlarmMsgWarn(List<String> response, Integer interval) {
        Random random = new Random();
        if (response.size() > 0) {
            JSONObject baseData = JSONObject.parseObject(response.get(0));
            List<AlarmMsgVo> alarmMsgVos = new ArrayList<>();
            if (baseData != null && baseData.get("data") != null) {
                alarmMsgVos = JSONObject.parseArray(baseData.get("data").toString(), AlarmMsgVo.class);
                //确实存在报警信息
                if (alarmMsgVos.size() > 0) {
                    List<AlarmMsgWarn> sidList = alarmMsgWarnMapper.selectList(new LambdaQueryWrapper<AlarmMsgWarn>()
                            .eq(AlarmMsgWarn::getType, 1)
                            .eq(AlarmMsgWarn::getIsDelete, 0).select(AlarmMsgWarn::getSid));
                    List<Long> collect = sidList.stream().map(AlarmMsgWarn::getSid).collect(Collectors.toList());

                    List<AlarmMsgWarn> alarmMsgList = new ArrayList<>();
                    List<Model> models = modelMapper.selectList(new LambdaQueryWrapper<Model>().eq(Model::getIsDelete, 0));
                    //sn号前
                    alarmMsgVos.forEach(o -> {
                        if (!collect.contains(o.getId())) {
                            AlarmMsgWarn alarmMsg = new AlarmMsgWarn();
                            if (models.size() > 0) {
                                models.forEach(m -> {
                                    if (m.getFaultCode().equals(o.getFaultCode())) {
                                        alarmMsg.setFaultLevel(m.getFaultLevel());
                                        alarmMsg.setFaultInfo(m.getAlgoName());
                                    }
                                });
                            }
                            alarmMsg.setSn(o.getSn());
                            alarmMsg.setFaultCode(o.getFaultCode());
                            LocalDateTime faultTime = DateUtil.Timestamp2LocalDateTime(o.getStartTime().getTime() + interval * 24 * 60 * 60 * 1000L);
                            alarmMsg.setFaultTime(faultTime);
                            alarmMsg.setFaultAdvice(o.getFaultAdvice());

                            if (alarmMsg.getFaultLevel() <= 3) {
                                alarmMsg.setFastCharge(BigDecimal.valueOf(random.nextInt(90)));
                            } else {
                                //数据类型)(最小值+Math.random()*(最大值-最小值+1)
                                int r = (int) (90 + Math.random() * (100 - 90 + 1));
                                alarmMsg.setFastCharge(BigDecimal.valueOf(r));
                            }
                            alarmMsg.setFaultStatus(0);
                            alarmMsg.setSid(o.getId());
                            alarmMsg.setType(1);
                            if (alarmMsg.getFaultLevel() != null && alarmMsg.getFaultLevel() > 3) {
                                alarmMsg.setUpdateBy(o.getUpdateBy());
                            }
                            alarmMsgList.add(alarmMsg);
                        }
                    });
                    if (alarmMsgList.size() > 0) {
                        alarmMsgList.forEach(o -> {
                            alarmMsgWarnMapper.insert(o);
                        });
                    }
                }
            }

            JSONObject baseData1 = JSONObject.parseObject(response.get(1));
            List<AlarmMsgVo> alarmMsgVos1 = new ArrayList<>();
            if (baseData1 != null && baseData1.get("data") != null) {
                alarmMsgVos1 = JSONObject.parseArray(baseData1.get("data").toString(), AlarmMsgVo.class);
                //确实存在报警信息
                if (alarmMsgVos1.size() > 0) {
                    Set<Long> sidSet = new HashSet<>();
                    alarmMsgVos1.forEach(o -> {
                        //flagSet.add(o.getSn() + o.getFaultCode() + o.getStartTime().getTime());
                        sidSet.add(o.getId());
                    });
                    List<AlarmMsgWarn> list = alarmMsgWarnMapper.selectList(new LambdaQueryWrapper<AlarmMsgWarn>().eq(AlarmMsgWarn::getType, 2).in(AlarmMsgWarn::getSid, sidSet));
                    List<AlarmMsgWarn> listFinal = new ArrayList<>();

                    List<AlarmMsgWarn> alarmMsgList = new ArrayList<>();
                    List<Model> models = modelMapper.selectList(new LambdaQueryWrapper<Model>().eq(Model::getIsDelete, 0));
                    //sn号前
                    alarmMsgVos1.forEach(o -> {
                        //故障表是否已经存在
                        AtomicBoolean had = new AtomicBoolean(false);
                        if (sidSet.size() > 0) {
                            if (sidSet.contains(o.getId())) {
                                list.forEach(m -> {
                                    if (m.getSid().equals(o.getId())) {
                                        //发生和完结时间中包含当前时间的
                                        LocalDateTime startTime = DateUtil.Timestamp2LocalDateTime(o.getStartTime().getTime() + interval * 24 * 60 * 60 * 1000L);
                                        LocalDateTime finishTime = DateUtil.Timestamp2LocalDateTime(o.getEndTime().getTime() + interval * 24 * 60 * 60 * 1000L);
                                        if (startTime.isBefore(LocalDateTime.now()) && finishTime.isAfter(LocalDateTime.now())) {
                                            m.setFaultStatus(1);
                                            m.setFinishTime(finishTime);
                                            m.setType(2);
                                            listFinal.add(m);
                                            had.set(true);
                                        }
                                    }
                                });
                            }
                        }
                        if (!had.get()) {
                            AlarmMsgWarn alarmMsg = new AlarmMsgWarn();
                            if (models.size() > 0) {
                                models.forEach(m -> {
                                    if (m.getFaultCode().equals(o.getFaultCode())) {
                                        alarmMsg.setFaultLevel(m.getFaultLevel());
                                        alarmMsg.setFaultInfo(m.getAlgoName());
                                    }
                                });
                            }
                            alarmMsg.setSn(o.getSn());
                            alarmMsg.setFaultCode(o.getFaultCode());
                            LocalDateTime faultTime = DateUtil.Timestamp2LocalDateTime(o.getStartTime().getTime() + interval * 24 * 60 * 60 * 1000L);
                            alarmMsg.setFaultTime(faultTime);
                            alarmMsg.setFaultAdvice(o.getFaultAdvice());
                            alarmMsg.setType(2);
                            if (alarmMsg.getFaultLevel() <= 3) {
                                alarmMsg.setFastCharge(BigDecimal.valueOf(random.nextInt(90)));
                            } else {
                                //数据类型)(最小值+Math.random()*(最大值-最小值+1)
                                int r = (int) (90 + Math.random() * (100 - 90 + 1));
                                alarmMsg.setFastCharge(BigDecimal.valueOf(r));
                            }
                            alarmMsg.setFaultStatus(0);
                            LocalDateTime finishTime = DateUtil.Timestamp2LocalDateTime(o.getEndTime().getTime() + interval * 24 * 60 * 60 * 1000L);
                            alarmMsg.setFinishTime(finishTime);
                            if (alarmMsg.getFaultLevel() != null && alarmMsg.getFaultLevel() > 3) {
                                alarmMsg.setUpdateBy(o.getUpdateBy());
                            }
                            alarmMsg.setSid(o.getId());
                            alarmMsg.setId(null);
                            alarmMsgList.add(alarmMsg);
                        }
                    });
                    if (alarmMsgList.size() > 0) {
                        this.saveBatch(alarmMsgList);
                    }
                    //更新状态
                    if (listFinal.size() > 0) {
                        this.updateBatchById(listFinal);
                    }
                }
            }
        }
    }

    /**
     * 故障信息
     */
    @Async
    public Future<String> getBatteryMsg(String url, String paramJson) {
        Map<String, String> mapHeaders = new HashMap<>();
        mapHeaders.put("token", tokenSecret);
        String response = okHttpCli.doPostTokenJson(url, paramJson, mapHeaders);
        Future<String> result = new AsyncResult<>(response);
        return result;
    }

    /**
     * 各省比例
     * @param province
     * @return java.math.BigDecimal
     */
    private BigDecimal provinceRate(String province) {
        long pro = deviceMapper.selectCount(new LambdaQueryWrapper<Device>().eq(Device::getProvince, province).eq(Device::getIsDelete, 0));
        long total = deviceMapper.selectCount(new LambdaQueryWrapper<Device>().eq(Device::getIsDelete, 0));
        return total > 0 ? BigDecimal.valueOf(pro).divide(BigDecimal.valueOf(total), 2, RoundingMode.HALF_UP) : BigDecimal.ZERO;
    }

}