package com.zhili.dashboard.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.zhili.dashboard.entity.Device;
import com.zhili.dashboard.entity.DeviceDuration;
import com.zhili.dashboard.mapper.DeviceDurationMapper;
import com.zhili.dashboard.req.TableInfoParam;
import com.zhili.dashboard.service.IDeviceDurationService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zhili.dashboard.utils.OkHttpCli;
import com.zhili.dashboard.vo.BatteryDataVo;
import com.zhili.dashboard.vo.DeviceDurationVo;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.AsyncResult;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Future;

/**
 * <p>
 * 电池状态统计信息 服务实现类
 * </p>
 *
 * @author zyg
 * @since 2022-07-05
 */
@Service
public class DeviceDurationServiceImpl extends ServiceImpl<DeviceDurationMapper, DeviceDuration> implements IDeviceDurationService {

    @Resource
    OkHttpCli okHttpCli;

    /**
     * 数据获取地址
     */
    private static final String BATTERY_URL = "/admin/v1/hzBattery";
    /**
     * 故障信息地址
     */
    private static final String SIGNATURE = "AD5C4EC001352FFAC3A03CBA787A7417";

    @Value("${openapi.token-secret}")
    private String tokenSecret;
    @Value("${openapi.url}")
    private String openApiUrl;

    /**
     * 故障信息
     */
    @Async
    public Future<String> getBatteryMsg(String url, String paramJson) {
        Map<String, String> mapHeaders = new HashMap<>();
        mapHeaders.put("token", tokenSecret);
        String response = okHttpCli.doPostTokenJson(url, paramJson, mapHeaders);
        Future<String> result = new AsyncResult<>(response);
        return result;
    }

    /**
     * 同步统计信息
     */
    @Override
    public void syncDuration() {
        QueryWrapper<DeviceDuration> queryWrapper = new QueryWrapper<>();
        queryWrapper.select("max(sid) as id");
        DeviceDuration duration = this.getOne(queryWrapper);

        long maxId = 0L;
        if (duration != null && duration.getId() != null) {
            maxId = duration.getId();
        }

        //获取电池信息
        List<String> response = new ArrayList<>();
        try {
            List<Future<String>> futures = new ArrayList<>();
            TableInfoParam param = new TableInfoParam();
            param.setSignature(SIGNATURE);
            param.setInterfaceName("析锂时长");
            param.setCondition("id>" + maxId);
            param.setTableName("algo_lipltd_ai");
            param.setFields("id,sn,cusum");

            futures.add(getBatteryMsg(openApiUrl + BATTERY_URL, JSON.toJSONString(param)));
            //死循环，每隔2000ms执行一次，判断一下这三个异步调用的方法是否全都执行完了。
            while (true) {
                //使用Future的isDone()方法返回该方法是否执行完成
                if (futures.get(0).isDone()) {
                    //如果异步方法全部执行完，跳出循环
                    break;
                }
                //每隔200毫秒判断一次
                Thread.sleep(200);
            }
            for (Future<String> future : futures) {
                String string = future.get();
                response.add(string);
            }
        } catch (Exception e) {
            return;
        }
        // 故障信息解析
        saveDuration(response);
    }

    @Transactional(rollbackFor = Exception.class)
    public void saveDuration(List<String> response) {
        if (response.size() > 0) {
            JSONObject baseData = JSONObject.parseObject(response.get(0));
            List<DeviceDurationVo> deviceDurationVos = new ArrayList<>();
            if (baseData != null && baseData.get("data") != null) {
                deviceDurationVos = JSONObject.parseArray(baseData.get("data").toString(), DeviceDurationVo.class);
                //确实存在报警信息
                if (deviceDurationVos.size() > 0) {
                    List<DeviceDuration> deviceList = new ArrayList<>();
                    //sn号前
                    deviceDurationVos.forEach(o -> {
                        DeviceDuration device = new DeviceDuration();
                        device.setSn(o.getSn());
                        device.setDuration(new BigDecimal(o.getCusum()));
                        device.setSid(o.getId());
                        deviceList.add(device);
                    });
                    if (deviceList.size() > 0) {
                        this.saveBatch(deviceList);
                    }
                }
            }
        }
    }
}
