package com.zhili.dashboard.utils;


import io.jsonwebtoken.SignatureAlgorithm;
import io.jsonwebtoken.Claims;
import io.jsonwebtoken.JwtBuilder;
import io.jsonwebtoken.Jwts;

import java.util.Date;

/**
 * jwt工具类
 * @author user
 */
public class JwtUtils {
    /**
     * 实例
     */
    private static JwtUtils instance;

    /**
     * 发行者
     */
    private String subObject = "owner";

    /**
     * 过期时间，默认7天
     */
    private long expired = 1000 * 60 * 60 * 24 * 7;

    /**
     * jwt构造
     */
    private static JwtBuilder jwtBuilder;

    /**
     * 密钥
     */
    private String secret = "secret";

    /**
     * 获取实例
     *
     * @return
     */
    public static JwtUtils getInstance() {
        if (instance == null) {
            instance = new JwtUtils();
        }
        jwtBuilder = Jwts.builder();
        return instance;
    }

    /**
     * 荷载信息(通常是一个User信息，还包括一些其他的元数据)
     *
     * @param key
     * @param val
     * @return
     */
    public JwtUtils setClaim(String key, Object val) {
        jwtBuilder.claim(key, val);
        return this;
    }

    /**
     * 生成 jwt token
     *
     * @return
     */
    public String generateToken() {
        String token = jwtBuilder
                .setSubject(subObject)
                //.claim("id","121") // 参数
                .setIssuedAt(new Date())
                .setExpiration(new Date(System.currentTimeMillis() + expired))
                .signWith(SignatureAlgorithm.HS256, secret)
//                .compressWith(CompressionCodecs.DEFLATE)// 对载荷进行压缩
                .compact(); // 压缩一下
        return token;
    }

    /**
     * 解析 token
     *
     * @param token
     * @return
     */
    public Claims check(String token) {
        try {
            return Jwts.parser().setSigningKey(secret).parseClaimsJws(token).getBody();
        } catch (Exception e) {
        }
        return null;
    }

    public String getSubObject() {
        return subObject;
    }

    /**
     * 设置发行者
     *
     * @param subObject
     * @return
     */
    public JwtUtils setSubObject(String subObject) {
        this.subObject = subObject;
        return this;
    }

    public long getExpired() {
        return expired;
    }

    /**
     * 设置过期时间
     *
     * @param expired
     * @return
     */
    public JwtUtils setExpired(long expired) {
        this.expired = expired;
        return this;
    }

    public String getSecret() {
        return secret;
    }

    /**
     * 设置密钥
     *
     * @param secret
     * @return
     */
    public JwtUtils setSecret(String secret) {
        this.secret = secret;
        return this;
    }
}